@@ -230,7 +230,9 @@ struct stat dir_stats;
 #endif
 
   if (mkdir(ESD_UNIX_SOCKET_DIR, ESD_UNIX_SOCKET_DIR_MODE) == 0) {
-    chmod(ESD_UNIX_SOCKET_DIR, ESD_UNIX_SOCKET_DIR_MODE);
+    if (chmod(ESD_UNIX_SOCKET_DIR, ESD_UNIX_SOCKET_DIR_MODE) != 0) {
+      return -1;
+    }
     return 0;
   }
   /* If mkdir failed with EEXIST, test if it is a directory with
@@ -255,7 +257,7 @@ struct stat dir_stats;
        * required, just a mode that isn't more permissive than the
        * one requested.
        */
-      if ( ~ESD_UNIX_SOCKET_DIR_MODE & dir_stats.st_mode)
+      if ( ~ESD_UNIX_SOCKET_DIR_MODE & (dir_stats.st_mode & ~S_IFMT))
 	updateMode = 1;
 #if defined(S_ISVTX)
       if ((dir_stats.st_mode & S_IWOTH) && !(dir_stats.st_mode & S_ISVTX))
@@ -482,6 +484,7 @@ int esd_server_resume(void)
 	    /* turn ourselves back on */
 	    esd_on_standby = 0;
 	    esd_on_autostandby = 0;
+	    esd_forced_standby = 0;
 	}
     }
 
@@ -594,6 +597,9 @@ int main ( int argc, char *argv[] )
 	    esd_use_tcpip = 1;
 	} else if ( !strcmp( argv[ arg ], "-public" ) ) {
 	    esd_public = 1;
+	} else if ( !strcmp( argv[ arg ], "-promiscuous" ) ) {
+	    esd_is_owned = 1;
+	    esd_is_locked = 0;
 	} else if ( !strcmp( argv[ arg ], "-terminate" ) ) {
 	    esd_terminate = 1;
 	} else if ( !strcmp( argv[ arg ], "-spawnpid" ) ) {
@@ -608,24 +614,27 @@ int main ( int argc, char *argv[] )
 		fprintf(stderr, "Esound version " VERSION "\n");
 		exit (0);
 	} else if ( !strcmp( argv[ arg ], "-h" ) || !strcmp( argv[ arg ], "--help" ) ) {
+	    fprintf( stderr, "Esound version " VERSION "\n\n");
 	    fprintf( stderr, "Usage: esd [options]\n\n" );
-	    fprintf( stderr, "  -d DEVICE   force esd to use sound device DEVICE\n" );
-	    fprintf( stderr, "  -b          run server in 8 bit sound mode\n" );
-	    fprintf( stderr, "  -r RATE     run server at sample rate of RATE\n" );
-	    fprintf( stderr, "  -as SECS    free audio device after SECS of inactivity\n" );
-	    fprintf( stderr, "  -unix       use unix domain sockets instead of tcp/ip\n" );
-	    fprintf( stderr, "  -tcp        use tcp/ip sockets instead of unix domain\n" );
-	    fprintf( stderr, "  -public     make tcp/ip access public (other than localhost)\n" );
-	    fprintf( stderr, "  -terminate  terminate esd daemone after last client exits\n" );
-	    fprintf( stderr, "  -nobeeps    disable startup beeps\n" );
-	    fprintf( stderr, "  -trust      start esd even if use of %s can be insecure\n",
+            fprintf( stderr, "  -v --version  print version information\n" );
+	    fprintf( stderr, "  -d DEVICE     force esd to use sound device DEVICE\n" );
+	    fprintf( stderr, "  -b            run server in 8 bit sound mode\n" );
+	    fprintf( stderr, "  -r RATE       run server at sample rate of RATE\n" );
+	    fprintf( stderr, "  -as SECS      free audio device after SECS of inactivity\n" );
+	    fprintf( stderr, "  -unix         use unix domain sockets instead of tcp/ip\n" );
+	    fprintf( stderr, "  -tcp          use tcp/ip sockets instead of unix domain\n" );
+	    fprintf( stderr, "  -public       make tcp/ip access public (other than localhost)\n" );
+	    fprintf( stderr, "  -promiscuous  start unlocked and owned (disable authenticaton) NOT RECOMMENDED\n" );
+	    fprintf( stderr, "  -terminate    terminate esd daemone after last client exits\n" );
+	    fprintf( stderr, "  -nobeeps      disable startup beeps\n" );
+	    fprintf( stderr, "  -trust        start esd even if use of %s can be insecure\n",
 		     ESD_UNIX_SOCKET_DIR );
 #ifdef ESDBG
-	    fprintf( stderr, "  -vt         enable trace diagnostic info\n" );
-	    fprintf( stderr, "  -vc         enable comms diagnostic info\n" );
-	    fprintf( stderr, "  -vm         enable mixer diagnostic info\n" );
+	    fprintf( stderr, "  -vt           enable trace diagnostic info\n" );
+	    fprintf( stderr, "  -vc           enable comms diagnostic info\n" );
+	    fprintf( stderr, "  -vm           enable mixer diagnostic info\n" );
 #endif
-	    fprintf( stderr, "  -port PORT  listen for connections at PORT (only for tcp/ip)\n" );
+	    fprintf( stderr, "  -port PORT   listen for connections at PORT (only for tcp/ip)\n" );
 	    fprintf( stderr, "  -bind ADDRESS binds to ADDRESS (only for tcp/ip)\n" );
 	    fprintf( stderr, "\nPossible devices are:  %s\n", esd_audio_devices() );
 	    exit( 0 );
@@ -693,7 +702,7 @@ int main ( int argc, char *argv[] )
     esd_audio_rate = 48000;
     ESD_AUDIO_STUFF;
     if ( esd_audio_open() < 0 ) {
-      fprintf(stderr, "Audio device open for 44.1Khz, stereo, 8bit failed\n"
+      fprintf(stderr, "Audio device open for 48Khz, stereo,16bit failed\n"
 	      "Trying 22.05Khz, 8bit stereo.\n");
       /* cant do defaults ... try 22.05 kkz 8bit stereo */
       esd_audio_format = ESD_BITS8 | ESD_STEREO;
@@ -797,6 +806,9 @@ int main ( int argc, char *argv[] )
 	}
     }
 
+    /* put some stuff in sound driver before pausing */
+    esd_audio_write( NULL, 0);
+
     /* pause the sound output */
     esd_audio_pause();
 
